import pytest

from program.models import Note, TimeSlot
from program.tests.factories import NoteFactory

pytestmark = pytest.mark.django_db


def url(note=None):
    if note:
        return f"/api/v1/notes/{note.id}/"
    else:
        return "/api/v1/notes/"


def note_data(timeslot: TimeSlot) -> dict[str, str | int]:
    return {"content": "CONTENT", "title": "TITLE", "timeslot_id": timeslot.id}


def test_read_notes_as_unauthenticated_user(api_client):
    NOTES = 3
    NoteFactory.create_batch(size=NOTES)

    response = api_client.get(url())

    assert response.status_code == 200
    assert len(response.data) == NOTES


@pytest.mark.django_db(transaction=True)
def test_create_note_for_owned_show_fails_if_it_exists(
    user_with_note_perms, api_client_note_perms, owned_show_once_timeslot_perms
):
    data = note_data(timeslot=owned_show_once_timeslot_perms)

    response = api_client_note_perms.post(url(), data=data)

    assert response.status_code == 400


def test_create_note_for_owned_show(
    user_with_note_perms, api_client_note_perms, owned_show_once_timeslot_perms
):
    Note.objects.get(pk=owned_show_once_timeslot_perms.note.id).delete()

    data = note_data(timeslot=owned_show_once_timeslot_perms)

    response = api_client_note_perms.post(url(), data=data)

    assert response.status_code == 201


def test_create_note_not_found_for_not_owned_show(
    user_with_note_perms, api_client_note_perms, show_once_timeslot
):
    data = note_data(timeslot=show_once_timeslot)

    response = api_client_note_perms.patch(url(note=show_once_timeslot.note), data=data)

    assert response.status_code == 404


def test_update_note_for_owned_show(
    user_with_note_perms, api_client_note_perms, owned_show_once_timeslot_perms
):
    update = note_data(timeslot=owned_show_once_timeslot_perms)

    response = api_client_note_perms.patch(
        url(note=owned_show_once_timeslot_perms.note), data=update
    )

    assert response.status_code == 200


def test_update_note_not_found_for_not_owned_show(
    user_with_note_perms, api_client_note_perms, once_timeslot
):
    update = note_data(timeslot=once_timeslot)

    response = api_client_note_perms.patch(url(note=once_timeslot.note), data=update)

    assert response.status_code == 404


def test_delete_note_for_owned_show(
    user_with_note_perms, api_client_note_perms, owned_show_once_timeslot_perms
):
    response = api_client_note_perms.delete(url(note=owned_show_once_timeslot_perms.note))

    assert response.status_code == 204


def test_delete_note_not_found_for_not_owned_show(
    user_with_note_perms, api_client_note_perms, once_timeslot
):
    response = api_client_note_perms.delete(url(note=once_timeslot.note))

    assert response.status_code == 404
